@@ -9,13 +9,13 @@ use std::{fmt, io, thread};
 
 #[cfg(test)]
 use once_cell::sync::Lazy;
-#[cfg(target_arch = "wasm32")]
+#[cfg(all(target_arch = "wasm32", feature = "wasmbind"))]
 use web_time::Instant;
 
 use crate::draw_target::ProgressDrawTarget;
 use crate::state::{AtomicPosition, BarState, ProgressFinish, Reset, TabExpandedString};
 use crate::style::ProgressStyle;
-use crate::{ProgressBarIter, ProgressIterator, ProgressState};
+use crate::{iter, ProgressBarIter, ProgressIterator, ProgressState};
 
 /// A progress bar or spinner
 ///
@@ -328,6 +328,11 @@ impl ProgressBar {
         state.update_estimate_and_draw(Instant::now());
     }
 
+    /// Sets the elapsed time for the progress bar
+    pub fn set_elapsed(&self, elapsed: Duration) {
+        self.state().state.started = Instant::now().checked_sub(elapsed).unwrap();
+    }
+
     /// Creates a new weak reference to this [`ProgressBar`]
     pub fn downgrade(&self) -> WeakProgressBar {
         WeakProgressBar {
@@ -493,6 +498,7 @@ impl ProgressBar {
         ProgressBarIter {
             progress: self.clone(),
             it: read,
+            seek_max: iter::SeekMax::default(),
         }
     }
 
@@ -514,6 +520,7 @@ impl ProgressBar {
         ProgressBarIter {
             progress: self.clone(),
             it: write,
+            seek_max: iter::SeekMax::default(),
         }
     }
 
@@ -540,6 +547,7 @@ impl ProgressBar {
         ProgressBarIter {
             progress: self.clone(),
             it: write,
+            seek_max: iter::SeekMax::default(),
         }
     }
 
@@ -563,6 +571,7 @@ impl ProgressBar {
         ProgressBarIter {
             progress: self.clone(),
             it: read,
+            seek_max: iter::SeekMax::default(),
         }
     }
 
@@ -585,6 +594,7 @@ impl ProgressBar {
         ProgressBarIter {
             progress: self.clone(),
             it: stream,
+            seek_max: iter::SeekMax::default(),
         }
     }
 
@@ -618,6 +628,11 @@ impl ProgressBar {
         self.state().state.elapsed()
     }
 
+    /// Returns the current tab width
+    pub fn tab_width(&self) -> usize {
+        self.state().tab_width
+    }
+
     /// Index in the `MultiState`
     pub(crate) fn index(&self) -> Option<usize> {
         self.state().draw_target.remote().map(|(_, idx)| idx)
