@@ -6,31 +6,28 @@
 
 use crate::*;
 
-use tiny_skia_path::{PathStroker, Scalar, ScreenIntRect, SCALAR_MAX};
+use tiny_skia_path::{PathStroker, Scalar, SCALAR_MAX};
 
-use crate::clip::SubClipMaskRef;
-use crate::pipeline::RasterPipelineBlitter;
+use crate::geom::ScreenIntRect;
+use crate::mask::SubMaskRef;
+use crate::pipeline::{RasterPipelineBlitter, RasterPipelineBuilder};
 use crate::pixmap::SubPixmapMut;
 use crate::scan;
 
+use crate::geom::IntSizeExt;
 #[cfg(all(not(feature = "std"), feature = "no-std-float"))]
 use tiny_skia_path::NoStdFloat;
 
 /// A path filling rule.
-#[derive(Copy, Clone, PartialEq, Debug)]
+#[derive(Copy, Clone, Default, PartialEq, Debug)]
 pub enum FillRule {
     /// Specifies that "inside" is computed by a non-zero sum of signed edge crossings.
+    #[default]
     Winding,
     /// Specifies that "inside" is computed by an odd number of edge crossings.
     EvenOdd,
 }
 
-impl Default for FillRule {
-    fn default() -> Self {
-        FillRule::Winding
-    }
-}
-
 /// Controls how a shape should be painted.
 #[derive(Clone, PartialEq, Debug)]
 pub struct Paint<'a> {
@@ -46,9 +43,18 @@ pub struct Paint<'a> {
 
     /// Enables anti-aliased painting.
     ///
-    /// Default: false
+    /// Default: true
     pub anti_alias: bool,
 
+    /// Colorspace for blending.
+    ///
+    /// This enables gamma correction during the blend operation.  While skia supports
+    /// full color-space conversions, we only support a few (simple) cases.  Note that
+    /// any color space other than Linear will force using the high-quality pipeline.
+    ///
+    /// Default: Linear
+    pub colorspace: ColorSpace,
+
     /// Forces the high quality/precision rendering pipeline.
     ///
     /// `tiny-skia`, just like Skia, has two rendering pipelines:
@@ -73,13 +79,14 @@ impl Default for Paint<'_> {
         Paint {
             shader: Shader::SolidColor(Color::BLACK),
             blend_mode: BlendMode::default(),
-            anti_alias: false,
+            anti_alias: true,
+            colorspace: ColorSpace::default(),
             force_hq_pipeline: false,
         }
     }
 }
 
-impl<'a> Paint<'a> {
+impl Paint<'_> {
     /// Sets a paint source to a solid color.
     pub fn set_color(&mut self, color: Color) {
         self.shader = Shader::SolidColor(color);
@@ -107,9 +114,9 @@ impl Pixmap {
         rect: Rect,
         paint: &Paint,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
-        self.as_mut().fill_rect(rect, paint, transform, clip_mask)
+        mask: Option<&Mask>,
+    ) {
+        self.as_mut().fill_rect(rect, paint, transform, mask);
     }
 
     /// Draws a filled path onto the pixmap.
@@ -121,10 +128,10 @@ impl Pixmap {
         paint: &Paint,
         fill_rule: FillRule,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
+        mask: Option<&Mask>,
+    ) {
         self.as_mut()
-            .fill_path(path, paint, fill_rule, transform, clip_mask)
+            .fill_path(path, paint, fill_rule, transform, mask);
     }
 
     /// Strokes a path.
@@ -136,10 +143,10 @@ impl Pixmap {
         paint: &Paint,
         stroke: &Stroke,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
+        mask: Option<&Mask>,
+    ) {
         self.as_mut()
-            .stroke_path(path, paint, stroke, transform, clip_mask)
+            .stroke_path(path, paint, stroke, transform, mask);
     }
 
     /// Draws a `Pixmap` on top of the current `Pixmap`.
@@ -152,14 +159,22 @@ impl Pixmap {
         pixmap: PixmapRef,
         paint: &PixmapPaint,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
+        mask: Option<&Mask>,
+    ) {
         self.as_mut()
-            .draw_pixmap(x, y, pixmap, paint, transform, clip_mask)
+            .draw_pixmap(x, y, pixmap, paint, transform, mask);
+    }
+
+    /// Applies a masks.
+    ///
+    /// See [`PixmapMut::apply_mask`](struct.PixmapMut.html#method.apply_mask) for details.
+    pub fn apply_mask(&mut self, mask: &Mask) {
+        self.as_mut().apply_mask(mask);
     }
 }
 
 impl PixmapMut<'_> {
+    // TODO: accept NonZeroRect?
     /// Draws a filled rectangle onto the pixmap.
     ///
     /// This function is usually slower than filling a rectangular path,
@@ -167,58 +182,59 @@ impl PixmapMut<'_> {
     /// clipping of horizontal/vertical edges.
     ///
     /// Used mainly to render a pixmap onto a pixmap.
-    ///
-    /// Returns `None` when there is nothing to fill or in case of a numeric overflow.
     pub fn fill_rect(
         &mut self,
         rect: Rect,
         paint: &Paint,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
+        mask: Option<&Mask>,
+    ) {
         // TODO: we probably can use tiler for rect too
         if transform.is_identity() && !DrawTiler::required(self.width(), self.height()) {
             // TODO: ignore rects outside the pixmap
 
             let clip = self.size().to_screen_int_rect(0, 0);
 
-            let clip_mask = clip_mask.map(|mask| mask.as_submask());
+            let mask = mask.map(|mask| mask.as_submask());
             let mut subpix = self.as_subpixmap();
-            let mut blitter = RasterPipelineBlitter::new(paint, clip_mask, &mut subpix)?;
+            let mut blitter = match RasterPipelineBlitter::new(paint, mask, &mut subpix) {
+                Some(v) => v,
+                None => return, // nothing to do, all good
+            };
 
             if paint.anti_alias {
-                scan::fill_rect_aa(&rect, &clip, &mut blitter)
+                scan::fill_rect_aa(&rect, &clip, &mut blitter);
             } else {
-                scan::fill_rect(&rect, &clip, &mut blitter)
+                scan::fill_rect(&rect, &clip, &mut blitter);
             }
         } else {
             let path = PathBuilder::from_rect(rect);
-            self.fill_path(&path, paint, FillRule::Winding, transform, clip_mask)
+            self.fill_path(&path, paint, FillRule::Winding, transform, mask);
         }
     }
 
     /// Draws a filled path onto the pixmap.
-    ///
-    /// Returns `None` when there is nothing to fill or in case of a numeric overflow.
     pub fn fill_path(
         &mut self,
         path: &Path,
         paint: &Paint,
         fill_rule: FillRule,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
+        mask: Option<&Mask>,
+    ) {
         if transform.is_identity() {
             // This is sort of similar to SkDraw::drawPath
 
             // Skip empty paths and horizontal/vertical lines.
             let path_bounds = path.bounds();
             if path_bounds.width().is_nearly_zero() || path_bounds.height().is_nearly_zero() {
-                return None;
+                log::warn!("empty paths and horizontal/vertical lines cannot be filled");
+                return;
             }
 
             if is_too_big_for_math(path) {
-                return None;
+                log::warn!("path coordinates are too big");
+                return;
             }
 
             // TODO: ignore paths outside the pixmap
@@ -229,14 +245,28 @@ impl PixmapMut<'_> {
 
                 for tile in tiler {
                     let ts = Transform::from_translate(-(tile.x() as f32), -(tile.y() as f32));
-                    path = path.transform(ts)?;
+                    path = match path.transform(ts) {
+                        Some(v) => v,
+                        None => {
+                            log::warn!("path transformation failed");
+                            return;
+                        }
+                    };
                     paint.shader.transform(ts);
 
                     let clip_rect = tile.size().to_screen_int_rect(0, 0);
-                    let mut subpix = self.subpixmap(tile.to_int_rect())?;
+                    let mut subpix = match self.subpixmap(tile.to_int_rect()) {
+                        Some(v) => v,
+                        None => continue, // technically unreachable
+                    };
+
+                    let submask = mask.and_then(|mask| mask.submask(tile.to_int_rect()));
+                    let mut blitter = match RasterPipelineBlitter::new(&paint, submask, &mut subpix)
+                    {
+                        Some(v) => v,
+                        None => continue, // nothing to do, all good
+                    };
 
-                    let submask = clip_mask.and_then(|mask| mask.submask(tile.to_int_rect()));
-                    let mut blitter = RasterPipelineBlitter::new(&paint, submask, &mut subpix)?;
                     // We're ignoring "errors" here, because `fill_path` will return `None`
                     // when rendering a tile that doesn't have a path on it.
                     // Which is not an error in this case.
@@ -247,29 +277,40 @@ impl PixmapMut<'_> {
                     }
 
                     let ts = Transform::from_translate(tile.x() as f32, tile.y() as f32);
-                    path = path.transform(ts)?;
+                    path = match path.transform(ts) {
+                        Some(v) => v,
+                        None => return, // technically unreachable
+                    };
                     paint.shader.transform(ts);
                 }
-
-                Some(())
             } else {
                 let clip_rect = self.size().to_screen_int_rect(0, 0);
-                let submask = clip_mask.map(|mask| mask.as_submask());
+                let submask = mask.map(|mask| mask.as_submask());
                 let mut subpix = self.as_subpixmap();
-                let mut blitter = RasterPipelineBlitter::new(paint, submask, &mut subpix)?;
+                let mut blitter = match RasterPipelineBlitter::new(paint, submask, &mut subpix) {
+                    Some(v) => v,
+                    None => return, // nothing to do, all good
+                };
+
                 if paint.anti_alias {
-                    scan::path_aa::fill_path(path, fill_rule, &clip_rect, &mut blitter)
+                    scan::path_aa::fill_path(path, fill_rule, &clip_rect, &mut blitter);
                 } else {
-                    scan::path::fill_path(path, fill_rule, &clip_rect, &mut blitter)
+                    scan::path::fill_path(path, fill_rule, &clip_rect, &mut blitter);
                 }
             }
         } else {
-            let path = path.clone().transform(transform)?;
+            let path = match path.clone().transform(transform) {
+                Some(v) => v,
+                None => {
+                    log::warn!("path transformation failed");
+                    return;
+                }
+            };
 
             let mut paint = paint.clone();
             paint.shader.transform(transform);
 
-            self.fill_path(&path, &paint, fill_rule, Transform::identity(), clip_mask)
+            self.fill_path(&path, &paint, fill_rule, Transform::identity(), mask)
         }
     }
 
@@ -292,17 +333,24 @@ impl PixmapMut<'_> {
         paint: &Paint,
         stroke: &Stroke,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
+        mask: Option<&Mask>,
+    ) {
         if stroke.width < 0.0 {
-            return None;
+            log::warn!("negative stroke width isn't allowed");
+            return;
         }
 
         let res_scale = PathStroker::compute_resolution_scale(&transform);
 
         let dash_path;
         let path = if let Some(ref dash) = stroke.dash {
-            dash_path = path.dash(dash, res_scale)?;
+            dash_path = match path.dash(dash, res_scale) {
+                Some(v) => v,
+                None => {
+                    log::warn!("path dashing failed");
+                    return;
+                }
+            };
             &dash_path
         } else {
             path
@@ -327,16 +375,31 @@ impl PixmapMut<'_> {
 
                 if !transform.is_identity() {
                     paint.shader.transform(transform);
-                    path = path.transform(transform)?;
+                    path = match path.transform(transform) {
+                        Some(v) => v,
+                        None => {
+                            log::warn!("path transformation failed");
+                            return;
+                        }
+                    };
                 }
 
                 for tile in tiler {
                     let ts = Transform::from_translate(-(tile.x() as f32), -(tile.y() as f32));
-                    path = path.transform(ts)?;
+                    path = match path.transform(ts) {
+                        Some(v) => v,
+                        None => {
+                            log::warn!("path transformation failed");
+                            return;
+                        }
+                    };
                     paint.shader.transform(ts);
 
-                    let mut subpix = self.subpixmap(tile.to_int_rect())?;
-                    let submask = clip_mask.and_then(|mask| mask.submask(tile.to_int_rect()));
+                    let mut subpix = match self.subpixmap(tile.to_int_rect()) {
+                        Some(v) => v,
+                        None => continue, // technically unreachable
+                    };
+                    let submask = mask.and_then(|mask| mask.submask(tile.to_int_rect()));
 
                     // We're ignoring "errors" here, because `stroke_hairline` will return `None`
                     // when rendering a tile that doesn't have a path on it.
@@ -344,26 +407,42 @@ impl PixmapMut<'_> {
                     Self::stroke_hairline(&path, &paint, stroke.line_cap, submask, &mut subpix);
 
                     let ts = Transform::from_translate(tile.x() as f32, tile.y() as f32);
-                    path = path.transform(ts)?;
+                    path = match path.transform(ts) {
+                        Some(v) => v,
+                        None => return,
+                    };
                     paint.shader.transform(ts);
                 }
-
-                Some(())
             } else {
                 let subpix = &mut self.as_subpixmap();
-                let submask = clip_mask.map(|mask| mask.as_submask());
+                let submask = mask.map(|mask| mask.as_submask());
                 if !transform.is_identity() {
                     paint.shader.transform(transform);
 
-                    let path = path.clone().transform(transform)?; // TODO: avoid clone
-                    Self::stroke_hairline(&path, &paint, stroke.line_cap, submask, subpix)
+                    // TODO: avoid clone
+                    let path = match path.clone().transform(transform) {
+                        Some(v) => v,
+                        None => {
+                            log::warn!("path transformation failed");
+                            return;
+                        }
+                    };
+
+                    Self::stroke_hairline(&path, &paint, stroke.line_cap, submask, subpix);
                 } else {
-                    Self::stroke_hairline(path, &paint, stroke.line_cap, submask, subpix)
+                    Self::stroke_hairline(path, &paint, stroke.line_cap, submask, subpix);
                 }
             }
         } else {
-            let path = path.stroke(stroke, res_scale)?;
-            self.fill_path(&path, paint, FillRule::Winding, transform, clip_mask)
+            let path = match path.stroke(stroke, res_scale) {
+                Some(v) => v,
+                None => {
+                    log::warn!("path stroking failed");
+                    return;
+                }
+            };
+
+            self.fill_path(&path, paint, FillRule::Winding, transform, mask);
         }
     }
 
@@ -372,23 +451,24 @@ impl PixmapMut<'_> {
         path: &Path,
         paint: &Paint,
         line_cap: LineCap,
-        clip_mask: Option<SubClipMaskRef>,
+        mask: Option<SubMaskRef>,
         pixmap: &mut SubPixmapMut,
-    ) -> Option<()> {
+    ) {
         let clip = pixmap.size.to_screen_int_rect(0, 0);
-
-        let mut blitter = RasterPipelineBlitter::new(paint, clip_mask, pixmap)?;
-
+        let mut blitter = match RasterPipelineBlitter::new(paint, mask, pixmap) {
+            Some(v) => v,
+            None => return, // nothing to do, all good
+        };
         if paint.anti_alias {
-            scan::hairline_aa::stroke_path(path, line_cap, &clip, &mut blitter)
+            scan::hairline_aa::stroke_path(path, line_cap, &clip, &mut blitter);
         } else {
-            scan::hairline::stroke_path(path, line_cap, &clip, &mut blitter)
+            scan::hairline::stroke_path(path, line_cap, &clip, &mut blitter);
         }
     }
 
     /// Draws a `Pixmap` on top of the current `Pixmap`.
     ///
-    /// We basically filling a rectangle with a `pixmap` pattern.
+    /// The same as filling a rectangle with a `pixmap` pattern.
     pub fn draw_pixmap(
         &mut self,
         x: i32,
@@ -396,8 +476,8 @@ impl PixmapMut<'_> {
         pixmap: PixmapRef,
         paint: &PixmapPaint,
         transform: Transform,
-        clip_mask: Option<&ClipMask>,
-    ) -> Option<()> {
+        mask: Option<&Mask>,
+    ) {
         let rect = pixmap.size().to_int_rect(x, y).to_rect();
 
         // TODO: SkSpriteBlitter
@@ -418,9 +498,46 @@ impl PixmapMut<'_> {
             blend_mode: paint.blend_mode,
             anti_alias: false,        // Skia doesn't use it too.
             force_hq_pipeline: false, // Pattern will use hq anyway.
+            colorspace: ColorSpace::default(),
         };
 
-        self.fill_rect(rect, &paint, transform, clip_mask)
+        self.fill_rect(rect, &paint, transform, mask);
+    }
+
+    /// Applies a masks.
+    ///
+    /// When a `Mask` is passed to drawing methods, it will be used to mask-out
+    /// content we're about to draw.
+    /// This method masks-out an already drawn content.
+    /// It's not as fast, but can be useful when a mask is not available during drawing.
+    ///
+    /// This method is similar to filling the whole pixmap with an another,
+    /// mask-like pixmap using the `DestinationOut` blend mode.
+    ///
+    /// `Mask` must have the same size as `Pixmap`. No transform or offset are allowed.
+    pub fn apply_mask(&mut self, mask: &Mask) {
+        if self.size() != mask.size() {
+            log::warn!("Pixmap and Mask are expected to have the same size");
+            return;
+        }
+
+        // Just a dummy.
+        let pixmap_src = PixmapRef::from_bytes(&[0, 0, 0, 0], 1, 1).unwrap();
+
+        let mut p = RasterPipelineBuilder::new();
+        p.push(pipeline::Stage::LoadMaskU8);
+        p.push(pipeline::Stage::LoadDestination);
+        p.push(pipeline::Stage::DestinationIn);
+        p.push(pipeline::Stage::Store);
+        let mut p = p.compile();
+        let rect = self.size().to_screen_int_rect(0, 0);
+        p.run(
+            &rect,
+            pipeline::AAMaskCtx::default(),
+            mask.as_submask().mask_ctx(),
+            pixmap_src,
+            &mut self.as_subpixmap(),
+        );
     }
 }
 
@@ -472,7 +589,7 @@ fn treat_as_hairline(paint: &Paint, stroke: &Stroke, mut ts: Transform) -> Optio
 /// small constants (e.g. 2,3,4). To try to preflight issues where these optionations could turn
 /// finite path values into infinities (or NaNs), we allow the upper drawing code to reject
 /// the path if its bounds (in device coordinates) is too close to max float.
-fn is_too_big_for_math(path: &Path) -> bool {
+pub(crate) fn is_too_big_for_math(path: &Path) -> bool {
     // This value is just a guess. smaller is safer, but we don't want to reject largish paths
     // that we don't have to.
     const SCALE_DOWN_TO_ALLOW_FOR_SMALL_MULTIPLIES: f32 = 0.25;
