@@ -48,14 +48,14 @@ use alloc::vec::Vec;
 
 use arrayvec::ArrayVec;
 
-use tiny_skia_path::{NormalizedF32, ScreenIntRect};
+use tiny_skia_path::NormalizedF32;
 
-use crate::{Color, LengthU32, PremultipliedColor, PremultipliedColorU8, SpreadMode};
+use crate::{Color, PremultipliedColor, PremultipliedColorU8, SpreadMode};
 use crate::{PixmapRef, Transform};
 
 pub use blitter::RasterPipelineBlitter;
 
-use crate::math::LENGTH_U32_ONE;
+use crate::geom::ScreenIntRect;
 use crate::pixmap::SubPixmapMut;
 use crate::wide::u32x8;
 
@@ -77,7 +77,10 @@ pub enum Stage {
     SeedShader,
     LoadDestination,
     Store,
+    LoadDestinationU8,
+    StoreU8,
     Gather,
+    LoadMaskU8,
     MaskU8,
     ScaleU8,
     LerpU8,
@@ -121,17 +124,34 @@ pub enum Stage {
     RepeatX1,
     Gradient,
     EvenlySpaced2StopGradient,
+    XYToUnitAngle,
     XYToRadius,
     XYTo2PtConicalFocalOnCircle,
     XYTo2PtConicalWellBehaved,
+    XYTo2PtConicalSmaller,
     XYTo2PtConicalGreater,
+    XYTo2PtConicalStrip,
+    Mask2PtConicalNan,
     Mask2PtConicalDegenerates,
     ApplyVectorMask,
+    Alter2PtConicalCompensateFocal,
+    Alter2PtConicalUnswap,
+    NegateX,
+    ApplyConcentricScaleBias,
+    GammaExpand2,
+    GammaExpandDestination2,
+    GammaCompress2,
+    GammaExpand22,
+    GammaExpandDestination22,
+    GammaCompress22,
+    GammaExpandSrgb,
+    GammaExpandDestinationSrgb,
+    GammaCompressSrgb,
 }
 
-pub const STAGES_COUNT: usize = Stage::ApplyVectorMask as usize + 1;
+pub const STAGES_COUNT: usize = Stage::GammaCompressSrgb as usize + 1;
 
-impl<'a> PixmapRef<'a> {
+impl PixmapRef<'_> {
     #[inline(always)]
     pub(crate) fn gather(&self, index: u32x8) -> [PremultipliedColorU8; highp::STAGE_WIDTH] {
         let index: [u32; 8] = bytemuck::cast(index);
@@ -149,7 +169,7 @@ impl<'a> PixmapRef<'a> {
     }
 }
 
-impl<'a> SubPixmapMut<'a> {
+impl SubPixmapMut<'_> {
     #[inline(always)]
     pub(crate) fn offset(&self, dx: usize, dy: usize) -> usize {
         self.real_width * dy + dx
@@ -161,6 +181,12 @@ impl<'a> SubPixmapMut<'a> {
         &mut self.pixels_mut()[offset..]
     }
 
+    #[inline(always)]
+    pub(crate) fn slice_mask_at_xy(&mut self, dx: usize, dy: usize) -> &mut [u8] {
+        let offset = self.offset(dx, dy);
+        &mut self.data[offset..]
+    }
+
     #[inline(always)]
     pub(crate) fn slice4_at_xy(
         &mut self,
@@ -178,6 +204,15 @@ impl<'a> SubPixmapMut<'a> {
     ) -> &mut [PremultipliedColorU8; lowp::STAGE_WIDTH] {
         arrayref::array_mut_ref!(self.pixels_mut(), self.offset(dx, dy), lowp::STAGE_WIDTH)
     }
+
+    #[inline(always)]
+    pub(crate) fn slice16_mask_at_xy(
+        &mut self,
+        dx: usize,
+        dy: usize,
+    ) -> &mut [u8; lowp::STAGE_WIDTH] {
+        arrayref::array_mut_ref!(self.data, self.offset(dx, dy), lowp::STAGE_WIDTH)
+    }
 }
 
 #[derive(Default, Debug)]
@@ -201,25 +236,16 @@ impl AAMaskCtx {
     }
 }
 
-#[derive(Copy, Clone, Debug)]
-pub struct ClipMaskCtx<'a> {
+#[derive(Copy, Clone, Debug, Default)]
+pub struct MaskCtx<'a> {
     pub data: &'a [u8],
-    pub stride: LengthU32,
+    pub real_width: u32,
 }
 
-impl Default for ClipMaskCtx<'_> {
-    fn default() -> Self {
-        ClipMaskCtx {
-            data: &[],
-            stride: LENGTH_U32_ONE,
-        }
-    }
-}
-
-impl ClipMaskCtx<'_> {
+impl MaskCtx<'_> {
     #[inline(always)]
     fn offset(&self, dx: usize, dy: usize) -> usize {
-        self.stride.get() as usize * dy + dx
+        self.real_width as usize * dy + dx
     }
 }
 
@@ -309,6 +335,7 @@ pub struct TwoPointConicalGradientCtx {
     // This context is used only in highp, where we use Tx4.
     pub mask: u32x8,
     pub p0: f32,
+    pub p1: f32,
 }
 
 #[derive(Copy, Clone, Default, Debug)]
@@ -400,6 +427,10 @@ impl RasterPipelineBuilder {
                     *fun = highp::load_dst_tail as highp::StageFn;
                 } else if highp::fn_ptr(*fun) == highp::fn_ptr(highp::store) {
                     *fun = highp::store_tail as highp::StageFn;
+                } else if highp::fn_ptr(*fun) == highp::fn_ptr(highp::load_dst_u8) {
+                    *fun = highp::load_dst_u8_tail as highp::StageFn;
+                } else if highp::fn_ptr(*fun) == highp::fn_ptr(highp::store_u8) {
+                    *fun = highp::store_u8_tail as highp::StageFn;
                 } else if highp::fn_ptr(*fun) == highp::fn_ptr(highp::source_over_rgba) {
                     // SourceOverRgba calls load/store manually, without the pipeline,
                     // therefore we have to switch it too.
@@ -429,6 +460,10 @@ impl RasterPipelineBuilder {
                     *fun = lowp::load_dst_tail as lowp::StageFn;
                 } else if lowp::fn_ptr(*fun) == lowp::fn_ptr(lowp::store) {
                     *fun = lowp::store_tail as lowp::StageFn;
+                } else if lowp::fn_ptr(*fun) == lowp::fn_ptr(lowp::load_dst_u8) {
+                    *fun = lowp::load_dst_u8_tail as lowp::StageFn;
+                } else if lowp::fn_ptr(*fun) == lowp::fn_ptr(lowp::store_u8) {
+                    *fun = lowp::store_u8_tail as lowp::StageFn;
                 } else if lowp::fn_ptr(*fun) == lowp::fn_ptr(lowp::source_over_rgba) {
                     // SourceOverRgba calls load/store manually, without the pipeline,
                     // therefore we have to switch it too.
@@ -467,8 +502,8 @@ impl RasterPipeline {
     pub fn run(
         &mut self,
         rect: &ScreenIntRect,
-        mask_ctx: AAMaskCtx,
-        clip_mask_ctx: ClipMaskCtx,
+        aa_mask_ctx: AAMaskCtx,
+        mask_ctx: MaskCtx,
         pixmap_src: PixmapRef,
         pixmap_dst: &mut SubPixmapMut,
     ) {
@@ -481,8 +516,8 @@ impl RasterPipeline {
                     functions.as_slice(),
                     tail_functions.as_slice(),
                     rect,
+                    aa_mask_ctx,
                     mask_ctx,
-                    clip_mask_ctx,
                     &mut self.ctx,
                     pixmap_src,
                     pixmap_dst,
@@ -496,8 +531,8 @@ impl RasterPipeline {
                     functions.as_slice(),
                     tail_functions.as_slice(),
                     rect,
+                    aa_mask_ctx,
                     mask_ctx,
-                    clip_mask_ctx,
                     &mut self.ctx,
                     // lowp doesn't support pattern, so no `pixmap_src` for it.
                     pixmap_dst,
@@ -520,6 +555,7 @@ mod blend_tests {
 
     use super::*;
     use crate::{BlendMode, Color, Pixmap, PremultipliedColorU8};
+    use crate::geom::IntSizeExt;
 
     macro_rules! test_blend {
         ($name:ident, $mode:expr, $is_highp:expr, $r:expr, $g:expr, $b:expr, $a:expr) => {
@@ -538,7 +574,7 @@ mod blend_tests {
                 p.push(Stage::Store);
                 let mut p = p.compile();
                 let rect = pixmap.size().to_screen_int_rect(0, 0);
-                p.run(&rect, AAMaskCtx::default(), ClipMaskCtx::default(), pixmap_src,
+                p.run(&rect, AAMaskCtx::default(), MaskCtx::default(), pixmap_src,
                       &mut pixmap.as_mut().as_subpixmap());
 
                 assert_eq!(
