@@ -66,6 +66,39 @@ impl From<FontStretch> for fontdb::Stretch {
     }
 }
 
+/// A font variation axis setting.
+///
+/// Used for variable fonts to specify axis values like weight, width, etc.
+#[derive(Clone, Copy, Debug)]
+pub struct FontVariation {
+    /// The 4-byte axis tag (e.g., b"wght" for weight).
+    pub tag: [u8; 4],
+    /// The axis value.
+    pub value: f32,
+}
+
+impl FontVariation {
+    /// Creates a new font variation.
+    pub fn new(tag: [u8; 4], value: f32) -> Self {
+        Self { tag, value }
+    }
+}
+
+impl PartialEq for FontVariation {
+    fn eq(&self, other: &Self) -> bool {
+        self.tag == other.tag && self.value.to_bits() == other.value.to_bits()
+    }
+}
+
+impl Eq for FontVariation {}
+
+impl std::hash::Hash for FontVariation {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.tag.hash(state);
+        self.value.to_bits().hash(state);
+    }
+}
+
 /// A font style property.
 #[derive(Clone, Copy, PartialEq, Eq, Debug, Hash)]
 pub enum FontStyle {
@@ -113,6 +146,7 @@ pub struct Font {
     pub(crate) style: FontStyle,
     pub(crate) stretch: FontStretch,
     pub(crate) weight: u16,
+    pub(crate) variations: Vec<FontVariation>,
 }
 
 impl Font {
@@ -137,6 +171,11 @@ impl Font {
     pub fn weight(&self) -> u16 {
         self.weight
     }
+
+    /// Font variation settings for variable fonts.
+    pub fn variations(&self) -> &[FontVariation] {
+        &self.variations
+    }
 }
 
 /// A dominant baseline property.
@@ -218,6 +257,24 @@ impl Default for LengthAdjust {
     }
 }
 
+/// A font optical sizing property.
+///
+/// Controls automatic adjustment of the `opsz` axis in variable fonts
+/// based on font size. Matches CSS `font-optical-sizing`.
+#[derive(Clone, Copy, PartialEq, Debug)]
+pub enum FontOpticalSizing {
+    /// Automatically set `opsz` to match font size (browser default).
+    Auto,
+    /// Do not automatically adjust `opsz`.
+    None,
+}
+
+impl Default for FontOpticalSizing {
+    fn default() -> Self {
+        Self::Auto
+    }
+}
+
 /// A text span decoration style.
 ///
 /// In SVG, text decoration and text it's applied to can have different styles.
@@ -281,6 +338,7 @@ pub struct TextSpan {
     pub(crate) font_size: NonZeroPositiveF32,
     pub(crate) small_caps: bool,
     pub(crate) apply_kerning: bool,
+    pub(crate) font_optical_sizing: FontOpticalSizing,
     pub(crate) decoration: TextDecoration,
     pub(crate) dominant_baseline: DominantBaseline,
     pub(crate) alignment_baseline: AlignmentBaseline,
@@ -346,6 +404,15 @@ impl TextSpan {
         self.apply_kerning
     }
 
+    /// Font optical sizing mode.
+    ///
+    /// When `Auto` (default), the `opsz` axis will be automatically set
+    /// to match the font size for variable fonts that support it.
+    /// This matches the CSS `font-optical-sizing: auto` behavior.
+    pub fn font_optical_sizing(&self) -> FontOpticalSizing {
+        self.font_optical_sizing
+    }
+
     /// A span decorations.
     pub fn decoration(&self) -> &TextDecoration {
         &self.decoration
