@@ -1,5 +1,6 @@
 //! Possible ZIP compression methods.
 
+use crate::cfg_if_expr;
 use core::fmt;
 use std::io;
 
@@ -25,7 +26,7 @@ pub enum CompressionMethod {
     #[cfg(feature = "deflate64")]
     Deflate64,
     /// Compress the file using BZIP2
-    #[cfg(feature = "bzip2")]
+    #[cfg(feature = "_bzip2_any")]
     Bzip2,
     /// Encrypted using AES.
     ///
@@ -33,7 +34,7 @@ pub enum CompressionMethod {
     /// or from `ZipFileData`.
     #[cfg(feature = "aes-crypto")]
     Aes,
-    /// Compress the file using ZStandard
+    /// Compress the file using `ZStandard`
     #[cfg(feature = "zstd")]
     Zstd,
     /// Compress the file using LZMA
@@ -51,7 +52,7 @@ pub enum CompressionMethod {
     /// Compress the file using XZ
     #[cfg(feature = "xz")]
     Xz,
-    /// Compress the file using PPMd
+    /// Compress the file using `PPMd`
     #[cfg(feature = "ppmd")]
     Ppmd,
     /// Unsupported compression method
@@ -109,9 +110,9 @@ impl CompressionMethod {
     #[cfg(not(feature = "deflate64"))]
     pub const DEFLATE64: Self = CompressionMethod::Unsupported(9);
     pub const PKWARE_IMPLODE: Self = CompressionMethod::Unsupported(10);
-    #[cfg(feature = "bzip2")]
+    #[cfg(feature = "_bzip2_any")]
     pub const BZIP2: Self = CompressionMethod::Bzip2;
-    #[cfg(not(feature = "bzip2"))]
+    #[cfg(not(feature = "_bzip2_any"))]
     pub const BZIP2: Self = CompressionMethod::Unsupported(12);
     #[cfg(not(feature = "lzma"))]
     pub const LZMA: Self = CompressionMethod::Unsupported(14);
@@ -143,10 +144,10 @@ impl CompressionMethod {
     #[cfg(feature = "_deflate-any")]
     pub const DEFAULT: Self = CompressionMethod::Deflated;
 
-    #[cfg(all(not(feature = "_deflate-any"), feature = "bzip2"))]
+    #[cfg(all(not(feature = "_deflate-any"), feature = "_bzip2_any"))]
     pub const DEFAULT: Self = CompressionMethod::Bzip2;
 
-    #[cfg(all(not(feature = "_deflate-any"), not(feature = "bzip2")))]
+    #[cfg(all(not(feature = "_deflate-any"), not(feature = "_bzip2_any")))]
     pub const DEFAULT: Self = CompressionMethod::Stored;
 }
 impl CompressionMethod {
@@ -169,7 +170,7 @@ impl CompressionMethod {
             8 => CompressionMethod::Deflated,
             #[cfg(feature = "deflate64")]
             9 => CompressionMethod::Deflate64,
-            #[cfg(feature = "bzip2")]
+            #[cfg(feature = "_bzip2_any")]
             12 => CompressionMethod::Bzip2,
             #[cfg(feature = "lzma")]
             14 => CompressionMethod::Lzma,
@@ -186,11 +187,12 @@ impl CompressionMethod {
         }
     }
 
-    /// Converts a u16 to its corresponding CompressionMethod
+    /// Converts a u16 to its corresponding `CompressionMethod`
     #[deprecated(
         since = "0.5.7",
         note = "use a constant to construct a compression method"
     )]
+    #[must_use]
     pub const fn from_u16(val: u16) -> CompressionMethod {
         Self::parse_from_u16(val)
     }
@@ -209,7 +211,7 @@ impl CompressionMethod {
             CompressionMethod::Deflated => 8,
             #[cfg(feature = "deflate64")]
             CompressionMethod::Deflate64 => 9,
-            #[cfg(feature = "bzip2")]
+            #[cfg(feature = "_bzip2_any")]
             CompressionMethod::Bzip2 => 12,
             #[cfg(feature = "aes-crypto")]
             CompressionMethod::Aes => 99,
@@ -226,11 +228,12 @@ impl CompressionMethod {
         }
     }
 
-    /// Converts a CompressionMethod to a u16
+    /// Converts a `CompressionMethod` to a u16
     #[deprecated(
         since = "0.5.7",
         note = "to match on other compression methods, use a constant"
     )]
+    #[must_use]
     pub const fn to_u16(self) -> u16 {
         self.serialize_to_u16()
     }
@@ -238,7 +241,10 @@ impl CompressionMethod {
 
 impl Default for CompressionMethod {
     fn default() -> Self {
-        Self::DEFAULT
+        cfg_if_expr! {
+            #[cfg(feature = "_deflate-any")] => CompressionMethod::Deflated,
+            _ => CompressionMethod::Stored
+        }
     }
 }
 
@@ -256,7 +262,7 @@ pub const SUPPORTED_COMPRESSION_METHODS: &[CompressionMethod] = &[
     CompressionMethod::Deflated,
     #[cfg(feature = "deflate64")]
     CompressionMethod::Deflate64,
-    #[cfg(feature = "bzip2")]
+    #[cfg(feature = "_bzip2_any")]
     CompressionMethod::Bzip2,
     #[cfg(feature = "zstd")]
     CompressionMethod::Zstd,
@@ -272,7 +278,7 @@ pub(crate) enum Decompressor<R: io::BufRead> {
     Deflated(flate2::bufread::DeflateDecoder<R>),
     #[cfg(feature = "deflate64")]
     Deflate64(deflate64::Deflate64Decoder<R>),
-    #[cfg(feature = "bzip2")]
+    #[cfg(feature = "_bzip2_any")]
     Bzip2(bzip2::bufread::BzDecoder<R>),
     #[cfg(feature = "zstd")]
     Zstd(zstd::Decoder<'static, R>),
@@ -313,7 +319,7 @@ impl<R: io::BufRead> io::Read for Decompressor<R> {
             Decompressor::Deflated(r) => r.read(buf),
             #[cfg(feature = "deflate64")]
             Decompressor::Deflate64(r) => r.read(buf),
-            #[cfg(feature = "bzip2")]
+            #[cfg(feature = "_bzip2_any")]
             Decompressor::Bzip2(r) => r.read(buf),
             #[cfg(feature = "zstd")]
             Decompressor::Zstd(r) => r.read(buf),
@@ -330,8 +336,14 @@ impl<R: io::BufRead> io::Read for Decompressor<R> {
                     // 5.8.8.1 LZMA Version Information & 5.8.8.2 LZMA Properties Size
                     let mut header = [0; 4];
                     reader.read_exact(&mut header)?;
-                    let _version_information = u16::from_le_bytes(header[0..2].try_into().unwrap());
-                    let properties_size = u16::from_le_bytes(header[2..4].try_into().unwrap());
+                    let _version_information =
+                        u16::from_le_bytes(header[0..2].try_into().map_err(|e| {
+                            std::io::Error::other(format!("Cannot transform header to u16: {e}"))
+                        })?);
+                    let properties_size =
+                        u16::from_le_bytes(header[2..4].try_into().map_err(|e| {
+                            std::io::Error::other(format!("Cannot transform header to u16: {e}"))
+                        })?);
                     if properties_size != 5 {
                         return Err(io::Error::new(
                             io::ErrorKind::InvalidInput,
@@ -342,7 +354,10 @@ impl<R: io::BufRead> io::Read for Decompressor<R> {
                     let mut props_data = [0; 5];
                     reader.read_exact(&mut props_data)?;
                     let props = props_data[0];
-                    let dict_size = u32::from_le_bytes(props_data[1..5].try_into().unwrap());
+                    let dict_size =
+                        u32::from_le_bytes(props_data[1..5].try_into().map_err(|e| {
+                            std::io::Error::other(format!("Cannot transform header to u32: {e}"))
+                        })?);
 
                     // We don't need to handle the end-of-stream marker here, since the LZMA reader
                     // stops at the end-of-stream marker OR when it has decoded uncompressed_size bytes, whichever comes first.
@@ -375,8 +390,8 @@ impl<R: io::BufRead> io::Read for Decompressor<R> {
                     reader.read_exact(&mut buffer)?;
                     let parameters = u16::from_le_bytes(buffer);
 
-                    let order = ((parameters & 0x0F) + 1) as u32;
-                    let memory_size = 1024 * 1024 * (((parameters >> 4) & 0xFF) + 1) as u32;
+                    let order = u32::from((parameters & 0x0F) + 1);
+                    let memory_size = 1024 * 1024 * u32::from(((parameters >> 4) & 0xFF) + 1);
                     let restoration_method = (parameters >> 12) & 0x0F;
 
                     let mut decompressor = ppmd_rust::Ppmd8Decoder::new(
@@ -436,7 +451,7 @@ impl<R: io::BufRead> Decompressor<R> {
             CompressionMethod::Deflate64 => {
                 Decompressor::Deflate64(deflate64::Deflate64Decoder::with_buffer(reader))
             }
-            #[cfg(feature = "bzip2")]
+            #[cfg(feature = "_bzip2_any")]
             CompressionMethod::Bzip2 => Decompressor::Bzip2(bzip2::bufread::BzDecoder::new(reader)),
             #[cfg(feature = "zstd")]
             CompressionMethod::Zstd => Decompressor::Zstd(zstd::Decoder::with_buffer(reader)?),
@@ -466,7 +481,7 @@ impl<R: io::BufRead> Decompressor<R> {
             _ => {
                 return Err(crate::result::ZipError::UnsupportedArchive(
                     "Compression method not supported",
-                ))
+                ));
             }
         })
     }
@@ -480,7 +495,7 @@ impl<R: io::BufRead> Decompressor<R> {
             Decompressor::Deflated(r) => r.into_inner(),
             #[cfg(feature = "deflate64")]
             Decompressor::Deflate64(r) => r.into_inner(),
-            #[cfg(feature = "bzip2")]
+            #[cfg(feature = "_bzip2_any")]
             Decompressor::Bzip2(r) => r.into_inner(),
             #[cfg(feature = "zstd")]
             Decompressor::Zstd(r) => r.finish(),
@@ -517,9 +532,9 @@ mod test {
 
     #[test]
     fn from_eq_to() {
-        for v in 0..(u16::MAX as u32 + 1) {
+        for v in 0..(u32::from(u16::MAX) + 1) {
             let from = CompressionMethod::parse_from_u16(v as u16);
-            let to = from.serialize_to_u16() as u32;
+            let to = u32::from(from.serialize_to_u16());
             assert_eq!(v, to);
         }
     }
