@@ -1,8 +1,10 @@
 #![macro_use]
 
-use crate::read::magic_finder::{Backwards, Forward, MagicFinder, OptimisticMagicFinder};
+use crate::extra_fields::UsedExtraField;
 use crate::read::ArchiveOffset;
-use crate::result::{invalid, ZipError, ZipResult};
+use crate::read::magic_finder::{Backwards, Forward, MagicFinder, OptimisticMagicFinder};
+use crate::result::{ZipError, ZipResult, invalid};
+use core::any::type_name;
 use core::mem;
 use core::slice;
 use std::io::{self, Read, Seek, Write};
@@ -51,6 +53,46 @@ impl Magic {
     pub const DATA_DESCRIPTOR_SIGNATURE: Self = Self::literal(0x08074b50);
 }
 
+/// Zip flags
+/// Stored as Little endian
+#[allow(unused)]
+#[rustfmt::skip]
+#[repr(u16)]
+#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+pub(crate) enum ZipFlags {
+    /// If set, indicates that the file is encrypted.
+    Encrypted                   = 0b0000_0000_0000_0001,
+    CompressionSetting          = 0b0000_0000_0000_0010,
+    CompressionSetting2         = 0b0000_0000_0000_0100,
+    /// If this bit is set, the fields crc-32, compressed size and uncompressed size are set to zero in the  local header.
+    /// The correct values are put in the data descriptor immediately following the compressed data.
+    UsingDataDescriptor         = 0b0000_0000_0000_1000,
+    /// Reserved for use with method 8, for enhanced deflating.
+    ReservedEnhancedDeflating   = 0b0000_0000_0001_0000,
+    /// If this bit is set, this indicates that the file is compressed patched data.
+    CompressedPatchedData       = 0b0000_0000_0010_0000,
+    /// Strong encryption.
+    /// If this bit is set, you MUST set the version needed to extract value to at least 50 and you MUST also set bit 0.
+    /// If AES encryption is used, the version needed to extract value MUST be at least 51.
+    StrongEncryption            = 0b0000_0000_0100_0000,
+    // bit 7 Currently unused   = 0b0000_0000_1000_0000;
+    // bit 8 Currently unused   = 0b0000_0001_0000_0000;
+    // bit 9 Currently unused   = 0b0000_0010_0000_0000;
+    // bit 10 Currently unused  = 0b0000_0100_0000_0000;
+
+    /// Language encoding flag (EFS).
+    /// If this bit is set, the filename and comment fields for this file MUST be encoded using UTF-8.
+    LanguageEncoding            = 0b0000_1000_0000_0000,
+    /// Reserved by PKWARE for enhanced compression.
+    ReservedEnhancedCompression = 0b0001_0000_0000_0000,
+    /// Set when encrypting the Central Directory to indicate selected data values in the Local Header are masked to hide their actual values.
+    Masked                      = 0b0010_0000_0000_0000,
+    /// Reserved by PKWARE for alternate streams.
+    ReservedAlternateStream     = 0b0100_0000_0000_0000,
+    /// Reserved by PKWARE.
+    Reserved                    = 0b1000_0000_0000_0000,
+}
+
 /// Similar to [`Magic`], but used for extra field tags as per section 4.5.3 of APPNOTE.TXT.
 #[derive(Copy, Clone, Debug, PartialOrd, Ord, PartialEq, Eq, Hash)]
 #[repr(transparent)]
@@ -85,7 +127,7 @@ impl ExtraFieldMagic {
         Self(u16::to_le(self.0))
     }
 
-    pub const ZIP64_EXTRA_FIELD_TAG: Self = Self::literal(0x0001);
+    pub const ZIP64_EXTRA_FIELD_TAG: Self = Self::literal(UsedExtraField::Zip64ExtendedInfo as u16);
 }
 
 /// The file size at which a ZIP64 record becomes necessary.
@@ -163,28 +205,44 @@ pub(crate) unsafe trait Pod: Copy + 'static {
 
     #[inline]
     fn as_bytes(&self) -> &[u8] {
-        unsafe { slice::from_raw_parts(self as *const Self as *const u8, mem::size_of::<Self>()) }
+        unsafe {
+            slice::from_raw_parts(
+                std::ptr::from_ref::<Self>(self).cast::<u8>(),
+                mem::size_of::<Self>(),
+            )
+        }
     }
 
     #[inline]
     fn as_bytes_mut(&mut self) -> &mut [u8] {
-        unsafe { slice::from_raw_parts_mut(self as *mut Self as *mut u8, mem::size_of::<Self>()) }
+        unsafe {
+            slice::from_raw_parts_mut(
+                std::ptr::from_mut::<Self>(self).cast::<u8>(),
+                mem::size_of::<Self>(),
+            )
+        }
     }
 }
 
 pub(crate) trait FixedSizeBlock: Pod {
-    const MAGIC: Magic;
+    type Magic: Copy + Eq;
+    const MAGIC: Self::Magic;
 
-    fn magic(self) -> Magic;
+    fn magic(self) -> Self::Magic;
 
     const WRONG_MAGIC_ERROR: ZipError;
 
     #[allow(clippy::wrong_self_convention)]
     fn from_le(self) -> Self;
 
-    fn parse<R: Read>(reader: &mut R) -> ZipResult<Self> {
+    fn parse<R: Read + ?Sized>(reader: &mut R) -> ZipResult<Self> {
         let mut block = Self::zeroed();
-        reader.read_exact(block.as_bytes_mut())?;
+        if let Err(e) = reader.read_exact(block.as_bytes_mut()) {
+            if e.kind() == io::ErrorKind::UnexpectedEof {
+                return Err(invalid!("Unexpected end of {}", type_name::<Self>()));
+            }
+            return Err(e.into());
+        }
         let block = Self::from_le(block);
 
         if block.magic() != Self::MAGIC {
@@ -195,7 +253,7 @@ pub(crate) trait FixedSizeBlock: Pod {
 
     fn to_le(self) -> Self;
 
-    fn write<T: Write>(self, writer: &mut T) -> ZipResult<()> {
+    fn write<T: Write + ?Sized>(self, writer: &mut T) -> ZipResult<()> {
         let block = self.to_le();
         writer.write_all(block.as_bytes())?;
         Ok(())
@@ -233,6 +291,7 @@ macro_rules! to_le {
 /* TODO: derive macro to generate these fields? */
 /// Implement `from_le()` and `to_le()`, providing the field specification to both macros
 /// and methods.
+#[macro_export]
 macro_rules! to_and_from_le {
     ($($args:tt),+ $(,)?) => {
         #[inline(always)]
@@ -264,6 +323,7 @@ pub(crate) struct Zip32CDEBlock {
 unsafe impl Pod for Zip32CDEBlock {}
 
 impl FixedSizeBlock for Zip32CDEBlock {
+    type Magic = Magic;
     const MAGIC: Magic = Magic::CENTRAL_DIRECTORY_END_SIGNATURE;
 
     #[inline(always)]
@@ -321,7 +381,7 @@ impl Zip32CentralDirectoryEnd {
         (block, zip_file_comment)
     }
 
-    pub fn parse<T: Read>(reader: &mut T) -> ZipResult<Zip32CentralDirectoryEnd> {
+    pub fn parse<T: Read + ?Sized>(reader: &mut T) -> ZipResult<Zip32CentralDirectoryEnd> {
         let Zip32CDEBlock {
             // magic,
             disk_number,
@@ -383,6 +443,7 @@ pub(crate) struct Zip64CDELocatorBlock {
 unsafe impl Pod for Zip64CDELocatorBlock {}
 
 impl FixedSizeBlock for Zip64CDELocatorBlock {
+    type Magic = Magic;
     const MAGIC: Magic = Magic::ZIP64_CENTRAL_DIRECTORY_END_LOCATOR_SIGNATURE;
 
     #[inline(always)]
@@ -407,7 +468,7 @@ pub(crate) struct Zip64CentralDirectoryEndLocator {
 }
 
 impl Zip64CentralDirectoryEndLocator {
-    pub fn parse<T: Read>(reader: &mut T) -> ZipResult<Zip64CentralDirectoryEndLocator> {
+    pub fn parse<T: Read + ?Sized>(reader: &mut T) -> ZipResult<Zip64CentralDirectoryEndLocator> {
         let Zip64CDELocatorBlock {
             // magic,
             disk_with_central_directory,
@@ -460,6 +521,7 @@ pub(crate) struct Zip64CDEBlock {
 unsafe impl Pod for Zip64CDEBlock {}
 
 impl FixedSizeBlock for Zip64CDEBlock {
+    type Magic = Magic;
     const MAGIC: Magic = Magic::ZIP64_CENTRAL_DIRECTORY_END_SIGNATURE;
 
     fn magic(self) -> Magic {
@@ -496,7 +558,10 @@ pub(crate) struct Zip64CentralDirectoryEnd {
 }
 
 impl Zip64CentralDirectoryEnd {
-    pub fn parse<T: Read>(reader: &mut T, max_size: u64) -> ZipResult<Zip64CentralDirectoryEnd> {
+    pub fn parse<T: Read + ?Sized>(
+        reader: &mut T,
+        max_size: u64,
+    ) -> ZipResult<Zip64CentralDirectoryEnd> {
         let Zip64CDEBlock {
             record_size,
             version_made_by,
@@ -517,7 +582,14 @@ impl Zip64CentralDirectoryEnd {
         }
 
         let mut zip_file_comment = vec![0u8; record_size as usize - 44].into_boxed_slice();
-        reader.read_exact(&mut zip_file_comment)?;
+        if let Err(e) = reader.read_exact(&mut zip_file_comment) {
+            if e.kind() == io::ErrorKind::UnexpectedEof {
+                return Err(invalid!(
+                    "EOCD64 extensible data sector exceeds file boundary"
+                ));
+            }
+            return Err(e.into());
+        }
 
         Ok(Self {
             record_size,
@@ -598,7 +670,7 @@ pub(crate) struct CentralDirectoryEndInfo {
 ///
 /// In the best case scenario (no prepended junk), this function will not backtrack
 /// in the reader.
-pub(crate) fn find_central_directory<R: Read + Seek>(
+pub(crate) fn find_central_directory<R: Read + Seek + ?Sized>(
     reader: &mut R,
     archive_offset: ArchiveOffset,
     end_exclusive: u64,
@@ -641,7 +713,7 @@ pub(crate) fn find_central_directory<R: Read + Seek>(
 
         let zip64_metadata = if eocd.may_be_zip64() {
             fn try_read_eocd64_locator(
-                reader: &mut (impl Read + Seek),
+                reader: &mut (impl Read + Seek + ?Sized),
                 eocd_offset: u64,
             ) -> ZipResult<(u64, Zip64CentralDirectoryEndLocator)> {
                 if eocd_offset < mem::size_of::<Zip64CDELocatorBlock>() as u64 {
@@ -651,10 +723,8 @@ pub(crate) fn find_central_directory<R: Read + Seek>(
                 let locator64_offset = eocd_offset - mem::size_of::<Zip64CDELocatorBlock>() as u64;
 
                 reader.seek(io::SeekFrom::Start(locator64_offset))?;
-                Ok((
-                    locator64_offset,
-                    Zip64CentralDirectoryEndLocator::parse(reader)?,
-                ))
+                let locator64 = Zip64CentralDirectoryEndLocator::parse(reader);
+                Ok((locator64_offset, locator64?))
             }
 
             try_read_eocd64_locator(reader, eocd_offset).ok()
@@ -664,7 +734,7 @@ pub(crate) fn find_central_directory<R: Read + Seek>(
 
         let Some((locator64_offset, locator64)) = zip64_metadata else {
             // Branch out for zip32
-            let relative_cd_offset = eocd.central_directory_offset as u64;
+            let relative_cd_offset = u64::from(eocd.central_directory_offset);
 
             // If the archive is empty, there is nothing more to be checked, the archive is correct.
             if eocd.number_of_files == 0 {
@@ -726,7 +796,7 @@ pub(crate) fn find_central_directory<R: Read + Seek>(
 
         // This was hidden inside a function to collect errors in a single place.
         // Once try blocks are stabilized, this can go away.
-        fn try_read_eocd64<R: Read + Seek>(
+        fn try_read_eocd64<R: Read + Seek + ?Sized>(
             reader: &mut R,
             locator64: &Zip64CentralDirectoryEndLocator,
             expected_length: u64,
@@ -818,7 +888,7 @@ mod test {
     use std::io::Cursor;
 
     use crate::{
-        result::{invalid, ZipError},
+        result::{ZipError, invalid},
         spec::{FixedSizeBlock, Magic, Pod},
     };
 
@@ -832,6 +902,7 @@ mod test {
     unsafe impl Pod for TestBlock {}
 
     impl FixedSizeBlock for TestBlock {
+        type Magic = Magic;
         const MAGIC: Magic = Magic::literal(0x01111);
 
         fn magic(self) -> Magic {
